function iqdata = LoRa_modulator(symbols, SF, PL, syncword, ovs)
%LORA_MODULATOR creates a LoRa signal
%   This function crates a LoRa signal for the vector symbols with the
%   spreading factor SF, the preamble length PL, the syncword, and the
%   oversampling factor ovs.

% Authors: Holger Arthaber, Harald Eigner
%          TU Wien (EMCE), www.emce.tuwien.ac.at

% This work is licensed under a Creative Commons Attribution 4.0 International 
% (CC BY 4.0), https://creativecommons.org/licenses/by/4.0/
% You are free to:
%  - Share — copy and redistribute the material in any medium or format
%  - Adapt — remix, transform, and build upon the material for any purpose, 
%    even commercially. 
% Under the following terms:
%  - Attribution — You must give appropriate credit, provide a link to the license, 
%    and indicate if changes were made. You may do so in any reasonable manner, but not 
%    in any way that suggests the licensor endorses you or your use. 

if ~isscalar(PL) || any(PL < 0) || PL ~= floor(PL)
    error('PL (preamble length) must be an integer value >= 0');
end

if ~isscalar(syncword) || syncword < 0 || syncword > 255 || syncword ~= floor(syncword)
    error('SYNCWORD must be an integer in the uint8-range 0...255');
end

iqdata = [];
phaseacc = 0;

%create preamble chirps
for preamble = 1 : PL
    [chirp, phaseacc] = lora_chirpsignal(0, SF, 'up', ovs, phaseacc);
    iqdata = [iqdata chirp];
end

%create syncword chirps
[chirp, phaseacc] = lora_chirpsignal(floor(syncword/16)*8 , SF, 'up', ovs, phaseacc);
iqdata = [iqdata chirp];
[chirp, phaseacc] = lora_chirpsignal(mod(syncword,16)*8 , SF, 'up', ovs, phaseacc);
iqdata = [iqdata chirp];

%create 2.25 frame delimiter downchirps
[chirp, phaseacc] = lora_chirpsignal(0, SF, 'down', ovs, phaseacc);
iqdata = [iqdata chirp];
[chirp, phaseacc] = lora_chirpsignal(0, SF, 'down', ovs, phaseacc);
iqdata = [iqdata chirp];
[chirp, phaseacc] = lora_chirpsignal(0, SF, 'down_quarter', ovs, phaseacc);
iqdata = [iqdata chirp];

%create chirps for the input symbols
for sym = symbols
    [chirp, phaseacc] = lora_chirpsignal(sym, SF, 'up', ovs, phaseacc);
    iqdata = [iqdata chirp];
end

