function [header_CRC_valid, CRC_on, header_CR, payload_len] = decode_header(header_decoded_bin)
%DECODE_HEADER decodes the header of a LoRa message
%   The function deodes the header from the first eight symbols of a LoRa
%   message. Ich checkt the validity of the header and returns the
%   information contained in the header:
%   header_CRC_valid    indicates a valid header
%   CRC_on              precense of a payload CRC
%   header_CR           code rate of the message payload
%   payload_len         length of the payload

% Authors: Holger Arthaber, Harald Eigner
%          TU Wien (EMCE), www.emce.tuwien.ac.at

% This work is licensed under a Creative Commons Attribution 4.0 International 
% (CC BY 4.0), https://creativecommons.org/licenses/by/4.0/
% You are free to:
%  - Share — copy and redistribute the material in any medium or format
%  - Adapt — remix, transform, and build upon the material for any purpose, 
%    even commercially. 
% Under the following terms:
%  - Attribution — You must give appropriate credit, provide a link to the license, 
%    and indicate if changes were made. You may do so in any reasonable manner, but not 
%    in any way that suggests the licensor endorses you or your use. 


if ~ismatrix(header_decoded_bin) || any(header_decoded_bin < 0, 'all') || any(header_decoded_bin > 1, 'all') || any(header_decoded_bin ~= floor(header_decoded_bin), 'all')
    error('Invalid Codeblock');
end

%Check matrix forheader CRC validation
CRC_check(1, 1:12)=logical([1 1 1 1 0 1 0 0 1 0 0 0]);
CRC_check(2, 1:12)=logical([1 1 1 0 1 0 1 0 0 1 0 0]);
CRC_check(3, 1:12)=logical([0 1 0 1 1 0 0 1 0 0 1 0]);
CRC_check(4, 1:12)=logical([1 0 0 0 0 1 1 1 0 0 0 1]);
CRC_check(5, 1:12)=logical([0 0 0 0 0 0 0 0 1 1 1 1]);

%Extract header from data
header = [header_decoded_bin(3, :) header_decoded_bin(2, :) header_decoded_bin(1, :)];
%Extract header CRC from data
header_CRC_received = dec2hex(bi2de([header_decoded_bin(5, :) header_decoded_bin(4, :)]), 2);

CRC_cal(1:5) = 0;
for idx = 1 : 5
    CRC_cal(idx) = mod(sum(header.* CRC_check(idx, :)), 2);  %Multiplication CRC with checkmatrix c*H'=0
end
header_CRC_cal = dec2hex(bi2de(CRC_cal), 2);

%Check validity of header CRC
if header_CRC_cal == header_CRC_received
    header_CRC_valid = 'true';
else
    header_CRC_valid = 'false';
end

%Extract data from header
CRC_on = bi2de(header_decoded_bin(3, 1));
header_CR = bi2de(header_decoded_bin(3, 2:4)) + 4;
payload_len = bi2de([header_decoded_bin(2, :) header_decoded_bin(1, :)]);
end