function B_de = deinterleave(B)
%DEINTERLEAVE deinterleaves the binary codeblock B
%   This function is the reverse of the diagonal interleaving function and
%   deinterleaves the codeblock B. It transoforms the binary matrix B of size
%   codeword length times symbol alphabet size (CR + 4 x N) to the binary
%   matrix B_de of size symbol alphabet times codeword length (N x CR + 4)

% Authors: Holger Arthaber, Harald Eigner
%          TU Wien (EMCE), www.emce.tuwien.ac.at

% This work is licensed under a Creative Commons Attribution 4.0 International 
% (CC BY 4.0), https://creativecommons.org/licenses/by/4.0/
% You are free to:
%  - Share — copy and redistribute the material in any medium or format
%  - Adapt — remix, transform, and build upon the material for any purpose, 
%    even commercially. 
% Under the following terms:
%  - Attribution — You must give appropriate credit, provide a link to the license, 
%    and indicate if changes were made. You may do so in any reasonable manner, but not 
%    in any way that suggests the licensor endorses you or your use. 



if ~ismatrix(B) || any(B < 0, 'all') || any(B > 1, 'all') || any(B ~= floor(B), 'all')
    error('Invalid Codeblock');
end

B = fliplr(B);
s = length(B(:, 1));
c = length(B(1, :));

B_de = zeros(c, s);
B_de2 = zeros(c, s);
B_de3 = B.';

%perform deinterleaving
for idx1 = 1 : c
    for idx2 = 1 : s
        B_de2(idx1, idx2) = B_de3(1 + mod( idx2 + idx1 - 2, c), idx2);
    end
end

for idx1 = 1 : c
    for idx2 = 1 : s
        B_de(idx1, idx2) = B_de2(c - (idx1 - 1), idx2);
    end
end
end