function codewords = encoder(B, CR)
%ENCODE Data encoding
%   This function encodes the input data from matrix B with code rate:
%   CR = 1 ... code rate 4/5
%   CR = 2 ... code rate 4/6
%   CR = 3 ... code rate 4/7
%   CR = 4 ... code rate 4/8

% Authors: Holger Arthaber, Harald Eigner
%          TU Wien (EMCE), www.emce.tuwien.ac.at

% This work is licensed under a Creative Commons Attribution 4.0 International 
% (CC BY 4.0), https://creativecommons.org/licenses/by/4.0/
% You are free to:
%  - Share — copy and redistribute the material in any medium or format
%  - Adapt — remix, transform, and build upon the material for any purpose, 
%    even commercially. 
% Under the following terms:
%  - Attribution — You must give appropriate credit, provide a link to the license, 
%    and indicate if changes were made. You may do so in any reasonable manner, but not 
%    in any way that suggests the licensor endorses you or your use. 


if ~ismatrix(B) || any(B < 0, 'all') || any(B > 1, 'all') || any(B ~= floor(B), 'all')
    error('Invalid Codeblock');
end
if ~isscalar(CR) || CR < 5 || CR > 8 || CR ~= floor(CR)
    error('Invalid Code Rate');
end

len = size(B, 1);
N = CR;
k = 4;
codewords = zeros(len, k);
P = zeros(k, N - k);
I = logical(eye(k));

%Create generator matrix G
if CR > 5
    for idx = 1 : 4
        P(idx, 1) =  xor(xor(I(idx, 1), I(idx, 2)), I(idx, 3));
        P(idx, 2) =  xor(xor(I(idx, 2), I(idx, 3)), I(idx, 4));
        P(idx, 3) =  xor(xor(I(idx, 1), I(idx, 2)), I(idx, 4));
        P(idx, 4) =  xor(xor(I(idx, 1), I(idx, 3)), I(idx, 4));
    end
    G=[I P(1 : 4, 1 : (CR - 4))];   %Create generator matrix for CR = 1 ... 3
elseif CR == 5
    for idx = 1:4
        P(idx, 1) =  xor(xor(I(idx, 1), I(idx, 2)), xor(I(idx, 3), I(idx, 4)));
    end
    G=[I P];    %Create generator matrix for CR = 4
end

%Generation of codewords
for idx1 = 1 : len
    for idx2 = 1 : N
        codewords(idx1, idx2 )= mod(sum(B(idx1, :).*G(:, idx2)'), 2);  %multiplication codewords with generator matrix c = u * G
    end
end

end

