function b = gray2bin(g)
%BIN2GRAY performs a Gray demapping of the binary vector g
%   The function performs a Gray demapping of the binary vector b according
%   to:
%   b_i = g_i            for i = 0
%   b_i = g_i xor g_i-1  else

% Authors: Holger Arthaber, Harald Eigner
%          TU Wien (EMCE), www.emce.tuwien.ac.at

% This work is licensed under a Creative Commons Attribution 4.0 International 
% (CC BY 4.0), https://creativecommons.org/licenses/by/4.0/
% You are free to:
%  - Share — copy and redistribute the material in any medium or format
%  - Adapt — remix, transform, and build upon the material for any purpose, 
%    even commercially. 
% Under the following terms:
%  - Attribution — You must give appropriate credit, provide a link to the license, 
%    and indicate if changes were made. You may do so in any reasonable manner, but not 
%    in any way that suggests the licensor endorses you or your use. 


if ~isvector(g)
    error('g must be a binary vector');
end

len = size(g, 2);
b(len) = g(len);

for i = 1 : len - 1
    x = xor(b(len - i + 1), g(len - i));
    b(len - i) = x;
end
end