function B_int = interleave(B)
%INTERLEAVE interleaves the binary codeblock B
%   This function performs the diagonal interleaving of the codeblock B. It transoforms 
%   the binary matrix B of size symbol alphabet times codeword length (N x CR + 4) to the binary
%   matrix B_int of size codeword length times symbol alphabet size (CR + 4 x N)

% Authors: Holger Arthaber, Harald Eigner
%          TU Wien (EMCE), www.emce.tuwien.ac.at

% This work is licensed under a Creative Commons Attribution 4.0 International 
% (CC BY 4.0), https://creativecommons.org/licenses/by/4.0/
% You are free to:
%  - Share — copy and redistribute the material in any medium or format
%  - Adapt — remix, transform, and build upon the material for any purpose, 
%    even commercially. 
% Under the following terms:
%  - Attribution — You must give appropriate credit, provide a link to the license, 
%    and indicate if changes were made. You may do so in any reasonable manner, but not 
%    in any way that suggests the licensor endorses you or your use. 



if ~ismatrix(B) || any(B < 0, 'all') || any(B > 1, 'all') || any(B ~= floor(B), 'all')
    error('Invalid Codeblock');
end

c = size(B, 1);
s = size(B, 2);
bin_interleaved1 = zeros(c, s);
B_int = zeros(c, s);

%perform interleaving
for idx1 = 1 : c
    for idx2 = 1 : s
        bin_interleaved1(idx1, idx2) = B(c - (idx1 - 1), idx2);
    end
end

for idx1 = 1 : c
    for idx2 = 1 : s
        B_int(idx1, idx2)=bin_interleaved1(1 + mod(idx1 - idx2, c), idx2);
    end
end
B_int = fliplr(B_int.');
end
