function [chirp, phaseacc] = lora_chirpsignal(symbol, SF, chirpmode, ovs, phaseacc)
%LORA_CHIRPSIGNAL creates a LoRa chirp
%   The function lora_chirpsignal creates a chirp representing the input
%   symbol with the spreading factor SF, oversampling factor ovs, starting 
%   phase paseacc, and the chirpmode:
%   chirpmode = "UP"            upchirp
%   chirpmode = "DOWN"          downchirp
%   chirpmode = "DOWN_QUATER"   downchirp with 1/4 symbol length.


% Authors: Holger Arthaber, Harald Eigner
%          TU Wien (EMCE), www.emce.tuwien.ac.at

% This work is licensed under a Creative Commons Attribution 4.0 International 
% (CC BY 4.0), https://creativecommons.org/licenses/by/4.0/
% You are free to:
%  - Share — copy and redistribute the material in any medium or format
%  - Adapt — remix, transform, and build upon the material for any purpose, 
%    even commercially. 
% Under the following terms:
%  - Attribution — You must give appropriate credit, provide a link to the license, 
%    and indicate if changes were made. You may do so in any reasonable manner, but not 
%    in any way that suggests the licensor endorses you or your use. 


N = 2 ^ SF;
NN = N * ovs;
fMin = -pi / ovs;
fMax = pi / ovs;
fStep = (2 * pi) / (N * ovs * ovs);
f0 = (2 * pi * symbol) / NN;
f = fMin + f0;

%define length for mode "DOWN_QUATER"
if isequal(upper(chirpmode), 'DOWN_QUARTER')
    NN = NN / 4;
end
chirp = zeros(1, NN);

%create upchirp 
if isequal(upper(chirpmode), 'UP') 
    for idx = 1 : NN
        if f > fMax 
            f =  f - (fMax - fMin);
        end
        phaseacc = phaseacc + f;
        chirp(idx) = exp(1i * phaseacc);
        f = f + fStep;
    end
end

%create downchirp
if isequal(upper(chirpmode), 'DOWN') || isequal(upper(chirpmode), 'DOWN_QUARTER')
    for idx = 1 : NN
       	if f > fMax 
            f =  f - (fMax - fMin);
        end
        phaseacc = phaseacc - f;
        chirp(idx) = exp(1i * phaseacc);        
        f = f + fStep;
    end
end
