% Test script

% NOTE: All these codes focus on the LoRa transmitter. 
% The demodulator/decoder codes are suboptimum

% Authors: Holger Arthaber, Harald Eigner
%          TU Wien (EMCE), www.emce.tuwien.ac.at

% This work is licensed under a Creative Commons Attribution 4.0 International 
% (CC BY 4.0), https://creativecommons.org/licenses/by/4.0/
% You are free to:
%  - Share — copy and redistribute the material in any medium or format
%  - Adapt — remix, transform, and build upon the material for any purpose, 
%    even commercially. 
% Under the following terms:
%  - Attribution — You must give appropriate credit, provide a link to the license, 
%    and indicate if changes were made. You may do so in any reasonable manner, but not 
%    in any way that suggests the licensor endorses you or your use. 


SF = 12;                % speading factor 7 ... 12
tx_payload = [1 2 3];   % test payload
CR = 1;                 % code rate 1 (4/5); 2 (4/6); 3 (4/7); 4 (4/8)
CRC_on = 1;             % activated CRC
DE = 1;                 % activated low data rate mode
PL = 8;                 % preamble length
syncword = 52;          % syncword
ovs = 10;               % oversampling factor
BW = 125000;            % bandwidth

% encode payload to LoRa symbols
tx_symbols = LoRa_encoder(tx_payload, SF, CR, CRC_on, DE);

% modulate symbols to get LoRa complex baseband signal
tx_signal = LoRa_modulator(tx_symbols, SF, PL, syncword, ovs);

% assume the rx signal quals the tx signal without noise/distortions
% the reason is that the LoRa_demodulator is a very simplified one, only
% intended for testing the LoRa signal generation
rx_signal = tx_signal;

% demodulate LoRa baseband signal to get LoRa symbols (assumes perfect
% synchronization in time and frequency). Even then, the demodulator is
% likely to fail
[rx_preamble, rx_symbols] = LoRa_demodulator(rx_signal, SF, BW, ovs, PL);

% decode LoRa symbols
[rx_payload, ~, ~, ~, ~] = LoRa_decoder(rx_symbols, SF, DE);

% check if the encoder works correctly
if ~isequal(tx_payload, rx_payload)
    warning('The decoded payload does not match the encoded one.');
end

