function crc16reg = payload_CRC(payload)
%PAYLOAD_CRC creates the CRC for a paload data
%   the function creates the LoRa CRC for a specific payload. A CCITT-based
%   CRC with the polynomial p(x) = x^16 xor x^12 xor x^5 xor 1 is used.

% Authors: Holger Arthaber, Harald Eigner
%          TU Wien (EMCE), www.emce.tuwien.ac.at

% This work is licensed under a Creative Commons Attribution 4.0 International 
% (CC BY 4.0), https://creativecommons.org/licenses/by/4.0/
% You are free to:
%  - Share — copy and redistribute the material in any medium or format
%  - Adapt — remix, transform, and build upon the material for any purpose, 
%    even commercially. 
% Under the following terms:
%  - Attribution — You must give appropriate credit, provide a link to the license, 
%    and indicate if changes were made. You may do so in any reasonable manner, but not 
%    in any way that suggests the licensor endorses you or your use. 

if ~isvector(payload) || any(payload < 0) || any(payload > 255) || any(payload ~= floor(payload))
    error('PAYLOAD must be a vector of integers in the uint8-rang 0...255');
end

crc16reg = 0;
%create CRC
for databyte = payload
    for cnt = 1:8
        crc16reg = bitxor(bitshift(crc16reg, 1, 'uint16'), hex2dec('1021') * (crc16reg >= 32768));
    end
    crc16reg = bitxor(crc16reg, databyte);
end
end